@@ -128,6 +128,7 @@ UninitializeTouch(InputInfoPtr pInfo)
 
     mtdev_close(proto_data->mtdev);
     proto_data->mtdev = NULL;
+    proto_data->num_touches = 0;
 }
 
 static void
@@ -149,6 +150,7 @@ InitializeTouch(InputInfoPtr pInfo)
     }
 
     proto_data->cur_slot = proto_data->mtdev->caps.slot.value;
+    proto_data->num_touches = 0;
 
     proto_data->last_mt_vals = calloc(priv->num_slots,
                                       sizeof(ValuatorMask *));
@@ -660,7 +662,7 @@ EventReadHwState(InputInfoPtr pInfo,
     SynapticsParameters *para = &priv->synpara;
     struct eventcomm_proto_data *proto_data = priv->proto_data;
 
-    SynapticsResetTouchHwState(hw);
+    SynapticsResetTouchHwState(hw, FALSE);
 
     /* Reset cumulative values if buttons were not previously pressed */
     if (!hw->left && !hw->right && !hw->middle)
